# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: identity/user_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from union.internal.identity import user_payload_pb2 as identity_dot_user__payload__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1bidentity/user_service.proto\x12\x11\x63loudidl.identity\x1a\x1cgoogle/api/annotations.proto\x1a\x1bidentity/user_payload.proto2\x9c\x06\n\x0bUserService\x12s\n\nCreateUser\x12$.cloudidl.identity.CreateUserRequest\x1a%.cloudidl.identity.CreateUserResponse\"\x18\x82\xd3\xe4\x93\x02\x12\x1a\r/users/api/v1:\x01*\x12t\n\x07GetUser\x12!.cloudidl.identity.GetUserRequest\x1a\".cloudidl.identity.GetUserResponse\"\"\x82\xd3\xe4\x93\x02\x1c\x12\x1a/users/api/v1/{id.subject}\x12\x80\x01\n\nDeleteUser\x12$.cloudidl.identity.DeleteUserRequest\x1a%.cloudidl.identity.DeleteUserResponse\"%\x82\xd3\xe4\x93\x02\x1f*\x1a/users/api/v1/{id.subject}:\x01*\x12m\n\tListUsers\x12#.cloudidl.identity.ListUsersRequest\x1a$.cloudidl.identity.ListUsersResponse\"\x15\x82\xd3\xe4\x93\x02\x0f\x12\r/users/api/v1\x12\x82\x01\n\x0eListUsersCount\x12(.cloudidl.identity.ListUsersCountRequest\x1a).cloudidl.identity.ListUsersCountResponse\"\x1b\x82\xd3\xe4\x93\x02\x15\x12\x13/users/api/v1/count\x12\xaa\x01\n\x16GetUserAndGroupsForOrg\x12\x30.cloudidl.identity.GetUserAndGroupsForOrgRequest\x1a\x31.cloudidl.identity.GetUserAndGroupsForOrgResponse\"+\x82\xd3\xe4\x93\x02%\x12#/users/api/v1/{organization}/groupsB\xbd\x01\n\x15\x63om.cloudidl.identityB\x10UserServiceProtoH\x02P\x01Z+github.com/unionai/cloud/gen/pb-go/identity\xa2\x02\x03\x43IX\xaa\x02\x11\x43loudidl.Identity\xca\x02\x11\x43loudidl\\Identity\xe2\x02\x1d\x43loudidl\\Identity\\GPBMetadata\xea\x02\x12\x43loudidl::Identityb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'identity.user_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\025com.cloudidl.identityB\020UserServiceProtoH\002P\001Z+github.com/unionai/cloud/gen/pb-go/identity\242\002\003CIX\252\002\021Cloudidl.Identity\312\002\021Cloudidl\\Identity\342\002\035Cloudidl\\Identity\\GPBMetadata\352\002\022Cloudidl::Identity'
  _USERSERVICE.methods_by_name['CreateUser']._options = None
  _USERSERVICE.methods_by_name['CreateUser']._serialized_options = b'\202\323\344\223\002\022\032\r/users/api/v1:\001*'
  _USERSERVICE.methods_by_name['GetUser']._options = None
  _USERSERVICE.methods_by_name['GetUser']._serialized_options = b'\202\323\344\223\002\034\022\032/users/api/v1/{id.subject}'
  _USERSERVICE.methods_by_name['DeleteUser']._options = None
  _USERSERVICE.methods_by_name['DeleteUser']._serialized_options = b'\202\323\344\223\002\037*\032/users/api/v1/{id.subject}:\001*'
  _USERSERVICE.methods_by_name['ListUsers']._options = None
  _USERSERVICE.methods_by_name['ListUsers']._serialized_options = b'\202\323\344\223\002\017\022\r/users/api/v1'
  _USERSERVICE.methods_by_name['ListUsersCount']._options = None
  _USERSERVICE.methods_by_name['ListUsersCount']._serialized_options = b'\202\323\344\223\002\025\022\023/users/api/v1/count'
  _USERSERVICE.methods_by_name['GetUserAndGroupsForOrg']._options = None
  _USERSERVICE.methods_by_name['GetUserAndGroupsForOrg']._serialized_options = b'\202\323\344\223\002%\022#/users/api/v1/{organization}/groups'
  _globals['_USERSERVICE']._serialized_start=110
  _globals['_USERSERVICE']._serialized_end=906
# @@protoc_insertion_point(module_scope)
