# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: secret/definition.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.common import identifier_pb2 as common_dot_identifier__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from union.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17secret/definition.proto\x12\x0f\x63loudidl.secret\x1a\x17\x63ommon/identifier.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x17validate/validate.proto\"d\n\nSecretSpec\x12#\n\x0cstring_value\x18\x01 \x01(\tH\x00R\x0bstringValue\x12#\n\x0c\x62inary_value\x18\x02 \x01(\x0cH\x00R\x0b\x62inaryValueB\x0c\n\x05value\x12\x03\xf8\x42\x01\"\x97\x01\n\x10SecretIdentifier\x12-\n\x04name\x18\x01 \x01(\tB\x19\xfa\x42\x16r\x14\x10\x01\x32\x10^[-a-zA-Z0-9_]+$R\x04name\x12\"\n\x0corganization\x18\x02 \x01(\tR\x0corganization\x12\x16\n\x06\x64omain\x18\x03 \x01(\tR\x06\x64omain\x12\x18\n\x07project\x18\x04 \x01(\tR\x07project\"\x93\x01\n\x0eSecretMetadata\x12=\n\x0c\x63reated_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0b\x63reatedTime\x12\x42\n\rsecret_status\x18\x02 \x01(\x0b\x32\x1d.cloudidl.secret.SecretStatusR\x0csecretStatus\"\xa2\x01\n\x0cSecretStatus\x12\x45\n\x0eoverall_status\x18\x01 \x01(\x0e\x32\x1e.cloudidl.secret.OverallStatusR\roverallStatus\x12K\n\x0e\x63luster_status\x18\x02 \x03(\x0b\x32$.cloudidl.secret.ClusterSecretStatusR\rclusterStatus\"\xa3\x01\n\x13\x43lusterSecretStatus\x12<\n\x07\x63luster\x18\x01 \x01(\x0b\x32\".cloudidl.common.ClusterIdentifierR\x07\x63luster\x12N\n\x0fpresence_status\x18\x02 \x01(\x0e\x32%.cloudidl.secret.SecretPresenceStatusR\x0epresenceStatus\"\x85\x01\n\x06Secret\x12\x31\n\x02id\x18\x01 \x01(\x0b\x32!.cloudidl.secret.SecretIdentifierR\x02id\x12H\n\x0fsecret_metadata\x18\x02 \x01(\x0b\x32\x1f.cloudidl.secret.SecretMetadataR\x0esecretMetadata*^\n\rOverallStatus\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x15\n\x11PARTIALLY_PRESENT\x10\x01\x12\x11\n\rFULLY_PRESENT\x10\x02\x12\x12\n\x0eUNKNOWN_STATUS\x10\x03*=\n\x14SecretPresenceStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07MISSING\x10\x01\x12\x0b\n\x07PRESENT\x10\x02\x42\xb0\x01\n\x13\x63om.cloudidl.secretB\x0f\x44\x65\x66initionProtoH\x02P\x01Z)github.com/unionai/cloud/gen/pb-go/secret\xa2\x02\x03\x43SX\xaa\x02\x0f\x43loudidl.Secret\xca\x02\x0f\x43loudidl\\Secret\xe2\x02\x1b\x43loudidl\\Secret\\GPBMetadata\xea\x02\x10\x43loudidl::Secretb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'secret.definition_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023com.cloudidl.secretB\017DefinitionProtoH\002P\001Z)github.com/unionai/cloud/gen/pb-go/secret\242\002\003CSX\252\002\017Cloudidl.Secret\312\002\017Cloudidl\\Secret\342\002\033Cloudidl\\Secret\\GPBMetadata\352\002\020Cloudidl::Secret'
  _SECRETSPEC.oneofs_by_name['value']._options = None
  _SECRETSPEC.oneofs_by_name['value']._serialized_options = b'\370B\001'
  _SECRETIDENTIFIER.fields_by_name['name']._options = None
  _SECRETIDENTIFIER.fields_by_name['name']._serialized_options = b'\372B\026r\024\020\0012\020^[-a-zA-Z0-9_]+$'
  _globals['_OVERALLSTATUS']._serialized_start=1000
  _globals['_OVERALLSTATUS']._serialized_end=1094
  _globals['_SECRETPRESENCESTATUS']._serialized_start=1096
  _globals['_SECRETPRESENCESTATUS']._serialized_end=1157
  _globals['_SECRETSPEC']._serialized_start=127
  _globals['_SECRETSPEC']._serialized_end=227
  _globals['_SECRETIDENTIFIER']._serialized_start=230
  _globals['_SECRETIDENTIFIER']._serialized_end=381
  _globals['_SECRETMETADATA']._serialized_start=384
  _globals['_SECRETMETADATA']._serialized_end=531
  _globals['_SECRETSTATUS']._serialized_start=534
  _globals['_SECRETSTATUS']._serialized_end=696
  _globals['_CLUSTERSECRETSTATUS']._serialized_start=699
  _globals['_CLUSTERSECRETSTATUS']._serialized_end=862
  _globals['_SECRET']._serialized_start=865
  _globals['_SECRET']._serialized_end=998
# @@protoc_insertion_point(module_scope)
