# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: common/identifier.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17\x63ommon/identifier.proto\x12\x0f\x63loudidl.common\x1a\x17validate/validate.proto\"~\n\x11ProjectIdentifier\x12+\n\x0corganization\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x0corganization\x12\x1f\n\x06\x64omain\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x06\x64omain\x12\x1b\n\x04name\x18\x03 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x04name\"T\n\x11\x43lusterIdentifier\x12\"\n\x0corganization\x18\x01 \x01(\tR\x0corganization\x12\x1b\n\x04name\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x04name\"O\n\x15\x43lusterPoolIdentifier\x12\"\n\x0corganization\x18\x01 \x01(\tR\x0corganization\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\"_\n\x17\x43lusterConfigIdentifier\x12+\n\x0corganization\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x0corganization\x12\x17\n\x02id\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x02id\"3\n\x0eUserIdentifier\x12!\n\x07subject\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x07subject\":\n\x15\x41pplicationIdentifier\x12!\n\x07subject\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x07subject\"Q\n\x0eRoleIdentifier\x12\"\n\x0corganization\x18\x01 \x01(\tR\x0corganization\x12\x1b\n\x04name\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x04name\"O\n\rOrgIdentifier\x12>\n\x04name\x18\x01 \x01(\tB*\xfa\x42\'r%\x10\x01\x18?2\x1f^[a-z0-9]([-a-z0-9]*[a-z0-9])?$R\x04name\"y\n\x18ManagedClusterIdentifier\x12\x1b\n\x04name\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x04name\x12:\n\x03org\x18\x03 \x01(\x0b\x32\x1e.cloudidl.common.OrgIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x03orgJ\x04\x08\x01\x10\x02\"S\n\x10PolicyIdentifier\x12\"\n\x0corganization\x18\x01 \x01(\tR\x0corganization\x12\x1b\n\x04name\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x04nameB\xb0\x01\n\x13\x63om.cloudidl.commonB\x0fIdentifierProtoH\x02P\x01Z)github.com/unionai/cloud/gen/pb-go/common\xa2\x02\x03\x43\x43X\xaa\x02\x0f\x43loudidl.Common\xca\x02\x0f\x43loudidl\\Common\xe2\x02\x1b\x43loudidl\\Common\\GPBMetadata\xea\x02\x10\x43loudidl::Commonb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'common.identifier_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023com.cloudidl.commonB\017IdentifierProtoH\002P\001Z)github.com/unionai/cloud/gen/pb-go/common\242\002\003CCX\252\002\017Cloudidl.Common\312\002\017Cloudidl\\Common\342\002\033Cloudidl\\Common\\GPBMetadata\352\002\020Cloudidl::Common'
  _PROJECTIDENTIFIER.fields_by_name['organization']._options = None
  _PROJECTIDENTIFIER.fields_by_name['organization']._serialized_options = b'\372B\004r\002\020\001'
  _PROJECTIDENTIFIER.fields_by_name['domain']._options = None
  _PROJECTIDENTIFIER.fields_by_name['domain']._serialized_options = b'\372B\004r\002\020\001'
  _PROJECTIDENTIFIER.fields_by_name['name']._options = None
  _PROJECTIDENTIFIER.fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _CLUSTERIDENTIFIER.fields_by_name['name']._options = None
  _CLUSTERIDENTIFIER.fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _CLUSTERCONFIGIDENTIFIER.fields_by_name['organization']._options = None
  _CLUSTERCONFIGIDENTIFIER.fields_by_name['organization']._serialized_options = b'\372B\004r\002\020\001'
  _CLUSTERCONFIGIDENTIFIER.fields_by_name['id']._options = None
  _CLUSTERCONFIGIDENTIFIER.fields_by_name['id']._serialized_options = b'\372B\004r\002\020\001'
  _USERIDENTIFIER.fields_by_name['subject']._options = None
  _USERIDENTIFIER.fields_by_name['subject']._serialized_options = b'\372B\004r\002\020\001'
  _APPLICATIONIDENTIFIER.fields_by_name['subject']._options = None
  _APPLICATIONIDENTIFIER.fields_by_name['subject']._serialized_options = b'\372B\004r\002\020\001'
  _ROLEIDENTIFIER.fields_by_name['name']._options = None
  _ROLEIDENTIFIER.fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _ORGIDENTIFIER.fields_by_name['name']._options = None
  _ORGIDENTIFIER.fields_by_name['name']._serialized_options = b'\372B\'r%\020\001\030?2\037^[a-z0-9]([-a-z0-9]*[a-z0-9])?$'
  _MANAGEDCLUSTERIDENTIFIER.fields_by_name['name']._options = None
  _MANAGEDCLUSTERIDENTIFIER.fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _MANAGEDCLUSTERIDENTIFIER.fields_by_name['org']._options = None
  _MANAGEDCLUSTERIDENTIFIER.fields_by_name['org']._serialized_options = b'\372B\005\212\001\002\020\001'
  _POLICYIDENTIFIER.fields_by_name['name']._options = None
  _POLICYIDENTIFIER.fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_PROJECTIDENTIFIER']._serialized_start=69
  _globals['_PROJECTIDENTIFIER']._serialized_end=195
  _globals['_CLUSTERIDENTIFIER']._serialized_start=197
  _globals['_CLUSTERIDENTIFIER']._serialized_end=281
  _globals['_CLUSTERPOOLIDENTIFIER']._serialized_start=283
  _globals['_CLUSTERPOOLIDENTIFIER']._serialized_end=362
  _globals['_CLUSTERCONFIGIDENTIFIER']._serialized_start=364
  _globals['_CLUSTERCONFIGIDENTIFIER']._serialized_end=459
  _globals['_USERIDENTIFIER']._serialized_start=461
  _globals['_USERIDENTIFIER']._serialized_end=512
  _globals['_APPLICATIONIDENTIFIER']._serialized_start=514
  _globals['_APPLICATIONIDENTIFIER']._serialized_end=572
  _globals['_ROLEIDENTIFIER']._serialized_start=574
  _globals['_ROLEIDENTIFIER']._serialized_end=655
  _globals['_ORGIDENTIFIER']._serialized_start=657
  _globals['_ORGIDENTIFIER']._serialized_end=736
  _globals['_MANAGEDCLUSTERIDENTIFIER']._serialized_start=738
  _globals['_MANAGEDCLUSTERIDENTIFIER']._serialized_end=859
  _globals['_POLICYIDENTIFIER']._serialized_start=861
  _globals['_POLICYIDENTIFIER']._serialized_end=944
# @@protoc_insertion_point(module_scope)
