# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: workspace/workspace_definition_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from union.internal.workspace import workspace_definition_payload_pb2 as workspace_dot_workspace__definition__payload__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,workspace/workspace_definition_service.proto\x12\x12\x63loudidl.workspace\x1a\x1cgoogle/api/annotations.proto\x1a,workspace/workspace_definition_payload.proto2\x92\x0b\n\x18WorkspaceRegistryService\x12\xaf\x01\n\x19\x43reateWorkspaceDefinition\x12\x34.cloudidl.workspace.CreateWorkspaceDefinitionRequest\x1a\x35.cloudidl.workspace.CreateWorkspaceDefinitionResponse\"%\x82\xd3\xe4\x93\x02\x1f\"\x1a/workspace_registry/api/v1:\x01*\x12\xdf\x01\n\x16GetWorkspaceDefinition\x12\x31.cloudidl.workspace.GetWorkspaceDefinitionRequest\x1a\x32.cloudidl.workspace.GetWorkspaceDefinitionResponse\"^\x90\x02\x01\x82\xd3\xe4\x93\x02U\x12S/workspace_registry/api/v1/{id.org}/{id.project}/{id.domain}/{id.name}/{id.version}\x12\x8d\x02\n\x1fUpdateWorkspaceDefinitionEntity\x12:.cloudidl.workspace.UpdateWorkspaceDefinitionEntityRequest\x1a;.cloudidl.workspace.UpdateWorkspaceDefinitionEntityResponse\"q\x82\xd3\xe4\x93\x02k\x1ai/workspace_registry/api/v1/entity/{entity.id.org}/{entity.id.project}/{entity.id.domain}/{entity.id.name}\x12\xe5\x01\n\x19UpdateWorkspaceDefinition\x12\x34.cloudidl.workspace.UpdateWorkspaceDefinitionRequest\x1a\x35.cloudidl.workspace.UpdateWorkspaceDefinitionResponse\"[\x82\xd3\xe4\x93\x02U\x1aS/workspace_registry/api/v1/{id.org}/{id.project}/{id.domain}/{id.name}/{id.version}\x12\xe3\x01\n\x1fListWorkspaceDefinitionEntities\x12:.cloudidl.workspace.ListWorkspaceDefinitionEntitiesRequest\x1a;.cloudidl.workspace.ListWorkspaceDefinitionEntitiesResponse\"G\x90\x02\x01\x82\xd3\xe4\x93\x02>\x12</workspace_registry/api/v1/entities/{org}/{project}/{domain}\x12\x83\x02\n\x18ListWorkspaceDefinitions\x12\x33.cloudidl.workspace.ListWorkspaceDefinitionsRequest\x1a\x34.cloudidl.workspace.ListWorkspaceDefinitionsResponse\"|\x90\x02\x01\x82\xd3\xe4\x93\x02s\x12\x33/workspace_registry/api/v1/{org}/{project}/{domain}Z<\x12:/workspace_registry/api/v1/{org}/{project}/{domain}/{name}B\xd2\x01\n\x16\x63om.cloudidl.workspaceB\x1fWorkspaceDefinitionServiceProtoH\x02P\x01Z,github.com/unionai/cloud/gen/pb-go/workspace\xa2\x02\x03\x43WX\xaa\x02\x12\x43loudidl.Workspace\xca\x02\x12\x43loudidl\\Workspace\xe2\x02\x1e\x43loudidl\\Workspace\\GPBMetadata\xea\x02\x13\x43loudidl::Workspaceb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'workspace.workspace_definition_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.cloudidl.workspaceB\037WorkspaceDefinitionServiceProtoH\002P\001Z,github.com/unionai/cloud/gen/pb-go/workspace\242\002\003CWX\252\002\022Cloudidl.Workspace\312\002\022Cloudidl\\Workspace\342\002\036Cloudidl\\Workspace\\GPBMetadata\352\002\023Cloudidl::Workspace'
  _WORKSPACEREGISTRYSERVICE.methods_by_name['CreateWorkspaceDefinition']._options = None
  _WORKSPACEREGISTRYSERVICE.methods_by_name['CreateWorkspaceDefinition']._serialized_options = b'\202\323\344\223\002\037\"\032/workspace_registry/api/v1:\001*'
  _WORKSPACEREGISTRYSERVICE.methods_by_name['GetWorkspaceDefinition']._options = None
  _WORKSPACEREGISTRYSERVICE.methods_by_name['GetWorkspaceDefinition']._serialized_options = b'\220\002\001\202\323\344\223\002U\022S/workspace_registry/api/v1/{id.org}/{id.project}/{id.domain}/{id.name}/{id.version}'
  _WORKSPACEREGISTRYSERVICE.methods_by_name['UpdateWorkspaceDefinitionEntity']._options = None
  _WORKSPACEREGISTRYSERVICE.methods_by_name['UpdateWorkspaceDefinitionEntity']._serialized_options = b'\202\323\344\223\002k\032i/workspace_registry/api/v1/entity/{entity.id.org}/{entity.id.project}/{entity.id.domain}/{entity.id.name}'
  _WORKSPACEREGISTRYSERVICE.methods_by_name['UpdateWorkspaceDefinition']._options = None
  _WORKSPACEREGISTRYSERVICE.methods_by_name['UpdateWorkspaceDefinition']._serialized_options = b'\202\323\344\223\002U\032S/workspace_registry/api/v1/{id.org}/{id.project}/{id.domain}/{id.name}/{id.version}'
  _WORKSPACEREGISTRYSERVICE.methods_by_name['ListWorkspaceDefinitionEntities']._options = None
  _WORKSPACEREGISTRYSERVICE.methods_by_name['ListWorkspaceDefinitionEntities']._serialized_options = b'\220\002\001\202\323\344\223\002>\022</workspace_registry/api/v1/entities/{org}/{project}/{domain}'
  _WORKSPACEREGISTRYSERVICE.methods_by_name['ListWorkspaceDefinitions']._options = None
  _WORKSPACEREGISTRYSERVICE.methods_by_name['ListWorkspaceDefinitions']._serialized_options = b'\220\002\001\202\323\344\223\002s\0223/workspace_registry/api/v1/{org}/{project}/{domain}Z<\022:/workspace_registry/api/v1/{org}/{project}/{domain}/{name}'
  _globals['_WORKSPACEREGISTRYSERVICE']._serialized_start=145
  _globals['_WORKSPACEREGISTRYSERVICE']._serialized_end=1571
# @@protoc_insertion_point(module_scope)
