# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: secret/payload.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.secret import definition_pb2 as secret_dot_definition__pb2
from union.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14secret/payload.proto\x12\x0f\x63loudidl.secret\x1a\x17secret/definition.proto\x1a\x17validate/validate.proto\"\x90\x01\n\x13\x43reateSecretRequest\x12;\n\x02id\x18\x01 \x01(\x0b\x32!.cloudidl.secret.SecretIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x02id\x12<\n\x0bsecret_spec\x18\x02 \x01(\x0b\x32\x1b.cloudidl.secret.SecretSpecR\nsecretSpec\"\x16\n\x14\x43reateSecretResponse\"\x90\x01\n\x13UpdateSecretRequest\x12;\n\x02id\x18\x01 \x01(\x0b\x32!.cloudidl.secret.SecretIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x02id\x12<\n\x0bsecret_spec\x18\x02 \x01(\x0b\x32\x1b.cloudidl.secret.SecretSpecR\nsecretSpec\"\x16\n\x14UpdateSecretResponse\"O\n\x10GetSecretRequest\x12;\n\x02id\x18\x01 \x01(\x0b\x32!.cloudidl.secret.SecretIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x02id\"D\n\x11GetSecretResponse\x12/\n\x06secret\x18\x01 \x01(\x0b\x32\x17.cloudidl.secret.SecretR\x06secret\"R\n\x13\x44\x65leteSecretRequest\x12;\n\x02id\x18\x01 \x01(\x0b\x32!.cloudidl.secret.SecretIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x02id\"\x16\n\x14\x44\x65leteSecretResponse\"\xc4\x02\n\x12ListSecretsRequest\x12\"\n\x0corganization\x18\x01 \x01(\tR\x0corganization\x12\x16\n\x06\x64omain\x18\x02 \x01(\tR\x06\x64omain\x12\x18\n\x07project\x18\x03 \x01(\tR\x07project\x12\x14\n\x05limit\x18\x04 \x01(\x05R\x05limit\x12\x14\n\x05token\x18\x05 \x01(\tR\x05token\x12g\n\x12per_cluster_tokens\x18\x06 \x03(\x0b\x32\x39.cloudidl.secret.ListSecretsRequest.PerClusterTokensEntryR\x10perClusterTokens\x1a\x43\n\x15PerClusterTokensEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\x8d\x02\n\x13ListSecretsResponse\x12\x31\n\x07secrets\x18\x01 \x03(\x0b\x32\x17.cloudidl.secret.SecretR\x07secrets\x12\x14\n\x05token\x18\x02 \x01(\tR\x05token\x12h\n\x12per_cluster_tokens\x18\x03 \x03(\x0b\x32:.cloudidl.secret.ListSecretsResponse.PerClusterTokensEntryR\x10perClusterTokens\x1a\x43\n\x15PerClusterTokensEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\xad\x01\n\x13\x63om.cloudidl.secretB\x0cPayloadProtoH\x02P\x01Z)github.com/unionai/cloud/gen/pb-go/secret\xa2\x02\x03\x43SX\xaa\x02\x0f\x43loudidl.Secret\xca\x02\x0f\x43loudidl\\Secret\xe2\x02\x1b\x43loudidl\\Secret\\GPBMetadata\xea\x02\x10\x43loudidl::Secretb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'secret.payload_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023com.cloudidl.secretB\014PayloadProtoH\002P\001Z)github.com/unionai/cloud/gen/pb-go/secret\242\002\003CSX\252\002\017Cloudidl.Secret\312\002\017Cloudidl\\Secret\342\002\033Cloudidl\\Secret\\GPBMetadata\352\002\020Cloudidl::Secret'
  _CREATESECRETREQUEST.fields_by_name['id']._options = None
  _CREATESECRETREQUEST.fields_by_name['id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _UPDATESECRETREQUEST.fields_by_name['id']._options = None
  _UPDATESECRETREQUEST.fields_by_name['id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _GETSECRETREQUEST.fields_by_name['id']._options = None
  _GETSECRETREQUEST.fields_by_name['id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _DELETESECRETREQUEST.fields_by_name['id']._options = None
  _DELETESECRETREQUEST.fields_by_name['id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _LISTSECRETSREQUEST_PERCLUSTERTOKENSENTRY._options = None
  _LISTSECRETSREQUEST_PERCLUSTERTOKENSENTRY._serialized_options = b'8\001'
  _LISTSECRETSRESPONSE_PERCLUSTERTOKENSENTRY._options = None
  _LISTSECRETSRESPONSE_PERCLUSTERTOKENSENTRY._serialized_options = b'8\001'
  _globals['_CREATESECRETREQUEST']._serialized_start=92
  _globals['_CREATESECRETREQUEST']._serialized_end=236
  _globals['_CREATESECRETRESPONSE']._serialized_start=238
  _globals['_CREATESECRETRESPONSE']._serialized_end=260
  _globals['_UPDATESECRETREQUEST']._serialized_start=263
  _globals['_UPDATESECRETREQUEST']._serialized_end=407
  _globals['_UPDATESECRETRESPONSE']._serialized_start=409
  _globals['_UPDATESECRETRESPONSE']._serialized_end=431
  _globals['_GETSECRETREQUEST']._serialized_start=433
  _globals['_GETSECRETREQUEST']._serialized_end=512
  _globals['_GETSECRETRESPONSE']._serialized_start=514
  _globals['_GETSECRETRESPONSE']._serialized_end=582
  _globals['_DELETESECRETREQUEST']._serialized_start=584
  _globals['_DELETESECRETREQUEST']._serialized_end=666
  _globals['_DELETESECRETRESPONSE']._serialized_start=668
  _globals['_DELETESECRETRESPONSE']._serialized_end=690
  _globals['_LISTSECRETSREQUEST']._serialized_start=693
  _globals['_LISTSECRETSREQUEST']._serialized_end=1017
  _globals['_LISTSECRETSREQUEST_PERCLUSTERTOKENSENTRY']._serialized_start=950
  _globals['_LISTSECRETSREQUEST_PERCLUSTERTOKENSENTRY']._serialized_end=1017
  _globals['_LISTSECRETSRESPONSE']._serialized_start=1020
  _globals['_LISTSECRETSRESPONSE']._serialized_end=1289
  _globals['_LISTSECRETSRESPONSE_PERCLUSTERTOKENSENTRY']._serialized_start=950
  _globals['_LISTSECRETSRESPONSE_PERCLUSTERTOKENSENTRY']._serialized_end=1017
# @@protoc_insertion_point(module_scope)
