# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: workspace/workspace_instance_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from union.internal.workspace import workspace_instance_payload_pb2 as workspace_dot_workspace__instance__payload__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*workspace/workspace_instance_service.proto\x12\x12\x63loudidl.workspace\x1a\x1cgoogle/api/annotations.proto\x1a*workspace/workspace_instance_payload.proto2\x99\x07\n\x18WorkspaceInstanceService\x12\xa6\x01\n\x16StartWorkspaceInstance\x12\x31.cloudidl.workspace.StartWorkspaceInstanceRequest\x1a\x32.cloudidl.workspace.StartWorkspaceInstanceResponse\"%\x82\xd3\xe4\x93\x02\x1f\"\x1a/workspace_instance/api/v1:\x01*\x12\xcc\x01\n\x14GetWorkspaceInstance\x12/.cloudidl.workspace.GetWorkspaceInstanceRequest\x1a\x30.cloudidl.workspace.GetWorkspaceInstanceResponse\"Q\x90\x02\x01\x82\xd3\xe4\x93\x02H\x12\x46/workspace_instance/api/v1/{id.org}/{id.project}/{id.domain}/{id.name}\x12\xa6\x01\n\x16ListWorkspaceInstances\x12\x31.cloudidl.workspace.ListWorkspaceInstancesRequest\x1a\x32.cloudidl.workspace.ListWorkspaceInstancesResponse\"%\x90\x02\x01\x82\xd3\xe4\x93\x02\x1c\x12\x1a/workspace_instance/api/v1\x12\xcf\x01\n\x15StopWorkspaceInstance\x12\x30.cloudidl.workspace.StopWorkspaceInstanceRequest\x1a\x31.cloudidl.workspace.StopWorkspaceInstanceResponse\"Q\x82\xd3\xe4\x93\x02K*F/workspace_instance/api/v1/{id.org}/{id.project}/{id.domain}/{id.name}:\x01*\x12\x89\x01\n\x17WatchWorkspaceInstances\x12\x32.cloudidl.workspace.WatchWorkspaceInstancesRequest\x1a\x33.cloudidl.workspace.WatchWorkspaceInstancesResponse\"\x03\x90\x02\x01\x30\x01\x42\xd0\x01\n\x16\x63om.cloudidl.workspaceB\x1dWorkspaceInstanceServiceProtoH\x02P\x01Z,github.com/unionai/cloud/gen/pb-go/workspace\xa2\x02\x03\x43WX\xaa\x02\x12\x43loudidl.Workspace\xca\x02\x12\x43loudidl\\Workspace\xe2\x02\x1e\x43loudidl\\Workspace\\GPBMetadata\xea\x02\x13\x43loudidl::Workspaceb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'workspace.workspace_instance_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.cloudidl.workspaceB\035WorkspaceInstanceServiceProtoH\002P\001Z,github.com/unionai/cloud/gen/pb-go/workspace\242\002\003CWX\252\002\022Cloudidl.Workspace\312\002\022Cloudidl\\Workspace\342\002\036Cloudidl\\Workspace\\GPBMetadata\352\002\023Cloudidl::Workspace'
  _WORKSPACEINSTANCESERVICE.methods_by_name['StartWorkspaceInstance']._options = None
  _WORKSPACEINSTANCESERVICE.methods_by_name['StartWorkspaceInstance']._serialized_options = b'\202\323\344\223\002\037\"\032/workspace_instance/api/v1:\001*'
  _WORKSPACEINSTANCESERVICE.methods_by_name['GetWorkspaceInstance']._options = None
  _WORKSPACEINSTANCESERVICE.methods_by_name['GetWorkspaceInstance']._serialized_options = b'\220\002\001\202\323\344\223\002H\022F/workspace_instance/api/v1/{id.org}/{id.project}/{id.domain}/{id.name}'
  _WORKSPACEINSTANCESERVICE.methods_by_name['ListWorkspaceInstances']._options = None
  _WORKSPACEINSTANCESERVICE.methods_by_name['ListWorkspaceInstances']._serialized_options = b'\220\002\001\202\323\344\223\002\034\022\032/workspace_instance/api/v1'
  _WORKSPACEINSTANCESERVICE.methods_by_name['StopWorkspaceInstance']._options = None
  _WORKSPACEINSTANCESERVICE.methods_by_name['StopWorkspaceInstance']._serialized_options = b'\202\323\344\223\002K*F/workspace_instance/api/v1/{id.org}/{id.project}/{id.domain}/{id.name}:\001*'
  _WORKSPACEINSTANCESERVICE.methods_by_name['WatchWorkspaceInstances']._options = None
  _WORKSPACEINSTANCESERVICE.methods_by_name['WatchWorkspaceInstances']._serialized_options = b'\220\002\001'
  _globals['_WORKSPACEINSTANCESERVICE']._serialized_start=141
  _globals['_WORKSPACEINSTANCESERVICE']._serialized_end=1062
# @@protoc_insertion_point(module_scope)
